@@ -65,7 +65,7 @@ func (x *GoSNMP) SendTrap(trap SnmpTrap) (result *SnmpPacket, err error) {
 		}
 
 		if trap.Variables[0].Type != TimeTicks {
-			now := uint32(time.Now().Unix())
+			now := uint32(time.Now().Unix()) //nolint:gosec
 			timetickPDU := SnmpPDU{Name: "1.3.6.1.2.1.1.3.0", Type: TimeTicks, Value: now}
 			// prepend timetickPDU
 			trap.Variables = append([]SnmpPDU{timetickPDU}, trap.Variables...)
@@ -441,17 +441,61 @@ func (t *TrapListener) debugTrapHandler(s *SnmpPacket, u *net.UDPAddr) {
 }
 
 // UnmarshalTrap unpacks the SNMP Trap.
-//
-// NOTE: the trap code is currently unreliable when working with snmpv3 - pull requests welcome
 func (x *GoSNMP) UnmarshalTrap(trap []byte, useResponseSecurityParameters bool) (result *SnmpPacket, err error) {
-	result = new(SnmpPacket)
+	// Get only the version from the header of the trap
+	version, _, err := x.unmarshalVersionFromHeader(trap, new(SnmpPacket))
+	if err != nil {
+		x.Logger.Printf("UnmarshalTrap version unmarshal: %s\n", err)
+		return nil, err
+	}
+	// If there are multiple users configured and the SNMP trap is v3, see which user has valid credentials
+	// by iterating through the list matching the identifier and seeing which credentials are authentic / can be used to decrypt
+	if x.TrapSecurityParametersTable != nil && version == Version3 {
+		identifier, err := x.getTrapIdentifier(trap)
+		if err != nil {
+			x.Logger.Printf("UnmarshalTrap V3 get trap identifier: %s\n", err)
+			return nil, err
+		}
+		secParamsList, err := x.TrapSecurityParametersTable.Get(identifier)
+		if err != nil {
+			x.Logger.Printf("UnmarshalTrap V3 get security parameters from table: %s\n", err)
+			return nil, err
+		}
+		for _, secParams := range secParamsList {
+			// Copy the trap and pass the security parameters to try to unmarshal with
+			cpTrap := make([]byte, len(trap))
+			copy(cpTrap, trap)
+			if result, err = x.unmarshalTrapBase(cpTrap, secParams.Copy(), true); err == nil {
+				return result, nil
+			}
+		}
+		return nil, fmt.Errorf("no credentials successfully unmarshaled trap: %w", err)
+	}
+	return x.unmarshalTrapBase(trap, nil, useResponseSecurityParameters)
+}
 
-	if x.SecurityParameters != nil {
-		err = x.SecurityParameters.InitSecurityKeys()
+func (x *GoSNMP) getTrapIdentifier(trap []byte) (string, error) {
+	// Initialize a packet with no auth/priv to unmarshal ID/key for security parameters to use
+	packet := new(SnmpPacket)
+	_, err := x.unmarshalHeader(trap, packet)
+	// Return err if no identifier was able to be parsed after unmarshaling
+	if err != nil && packet.SecurityParameters.getIdentifier() == "" {
+		return "", err
+	}
+	return packet.SecurityParameters.getIdentifier(), nil
+}
+
+func (x *GoSNMP) unmarshalTrapBase(trap []byte, sp SnmpV3SecurityParameters, useResponseSecurityParameters bool) (*SnmpPacket, error) {
+	result := new(SnmpPacket)
+
+	if x.SecurityParameters != nil && sp == nil {
+		err := x.SecurityParameters.InitSecurityKeys()
 		if err != nil {
 			return nil, err
 		}
 		result.SecurityParameters = x.SecurityParameters.Copy()
+	} else {
+		result.SecurityParameters = sp
 	}
 
 	cursor, err := x.unmarshalHeader(trap, result)
