@@ -58,6 +58,9 @@ type SnmpV3SecurityParameters interface {
 	isAuthentic(packetBytes []byte, packet *SnmpPacket) (bool, error)
 	encryptPacket(scopedPdu []byte) ([]byte, error)
 	decryptPacket(packet []byte, cursor int) ([]byte, error)
+	getIdentifier() string
+	getLogger() Logger
+	setLogger(log Logger)
 }
 
 func (x *GoSNMP) validateParametersV3() error {
@@ -364,7 +367,7 @@ func (x *GoSNMP) unmarshalV3Header(packet []byte,
 	}
 
 	if MsgID, ok := rawMsgID.(int); ok {
-		response.MsgID = uint32(MsgID)
+		response.MsgID = uint32(MsgID) //nolint:gosec
 		x.Logger.Printf("Parsed message ID %d", MsgID)
 	}
 
@@ -378,7 +381,7 @@ func (x *GoSNMP) unmarshalV3Header(packet []byte,
 	}
 
 	if MsgMaxSize, ok := rawMsgMaxSize.(int); ok {
-		response.MsgMaxSize = uint32(MsgMaxSize)
+		response.MsgMaxSize = uint32(MsgMaxSize) //nolint:gosec
 		x.Logger.Printf("Parsed message max size %d", MsgMaxSize)
 	}
 
@@ -406,7 +409,7 @@ func (x *GoSNMP) unmarshalV3Header(packet []byte,
 	}
 
 	if SecModel, ok := rawSecModel.(int); ok {
-		response.SecurityModel = SnmpV3SecurityModel(SecModel)
+		response.SecurityModel = SnmpV3SecurityModel(SecModel) //nolint:gosec
 		x.Logger.Printf("Parsed security model %d", SecModel)
 	}
 
